@@ -5,8 +5,7 @@ import math
 import datetime
 import warnings
 from sklearn.base import BaseEstimator
-from sklearn.decomposition import TruncatedSVD
-from sklearn.decomposition import PCA
+from sklearn.decomposition import TruncatedSVD, PCA
 from sklearn import preprocessing
 from annoy import AnnoyIndex
 
@@ -74,13 +73,13 @@ def hamming_dist(x1, x2):
 
 @numba.njit()
 def calculate_dist(x1, x2, distance_index):
-    if distance_index == 0:  # euclidean
+    if distance_index == 0: # euclidean
         return euclid_dist(x1, x2)
-    elif distance_index == 1:  # manhattan
+    elif distance_index == 1: # manhattan
         return manhattan_dist(x1, x2)
-    elif distance_index == 2:  # angular
+    elif distance_index == 2: # angular
         return angular_dist(x1, x2)
-    elif distance_index == 3:  # hamming
+    elif distance_index == 3: # hamming
         return hamming_dist(x1, x2)
 
 
@@ -140,8 +139,7 @@ def sample_MN_pair(X, n_MN, option=0):
             sampled = np.random.randint(0, n, 6)
             dist_list = np.empty((6), dtype=np.float32)
             for t in range(sampled.shape[0]):
-                dist_list[t] = calculate_dist(
-                    X[i], X[sampled[t]], distance_index=option)
+                dist_list[t] = calculate_dist(X[i], X[sampled[t]], distance_index=option)
             min_dic = np.argmin(dist_list)
             dist_list = np.delete(dist_list, [min_dic])
             sampled = np.delete(sampled, [min_dic])
@@ -159,12 +157,11 @@ def sample_MN_pair_deterministic(X, n_MN, random_state, option=0):
     for i in numba.prange(n):
         for jj in range(n_MN):
             # Shifting the seed to prevent sampling the same pairs
-            np.random.seed(random_state + i * n_MN + jj)
+            np.random.seed(random_state + i * n_MN + jj) 
             sampled = np.random.randint(0, n, 6)
             dist_list = np.empty((6), dtype=np.float32)
             for t in range(sampled.shape[0]):
-                dist_list[t] = calculate_dist(
-                    X[i], X[sampled[t]], distance_index=option)
+                dist_list[t] = calculate_dist(X[i], X[sampled[t]], distance_index=option)
             min_dic = np.argmin(dist_list)
             dist_list = np.delete(dist_list, [min_dic])
             sampled = np.delete(sampled, [min_dic])
@@ -273,7 +270,6 @@ def pacmap_grad(Y, pair_neighbors, pair_MN, pair_FP, w_neighbors, w_MN, w_FP):
         for d in range(dim):
             grad[i, d] -= w1 * y_ij[d]
             grad[j, d] += w1 * y_ij[d]
-
     grad[-1, 0] = loss.sum()
     return grad
 
@@ -322,10 +318,10 @@ def find_weight(w_MN_init, itr):
     return w_MN, w_neighbors, w_FP
 
 
-def preprocess_X(X, distance, apply_pca, verbose, seed, high_dim):
+def preprocess_X(X, distance, apply_pca, verbose, seed, high_dim, low_dim):
     tsvd = None
     pca_solution = False
-    if high_dim > 100 and apply_pca:
+    if distance != "hamming" and high_dim > 100 and apply_pca:
         X -= np.mean(X, axis=0)
         tsvd = TruncatedSVD(n_components=100, random_state=seed)
         X = tsvd.fit_transform(X)
@@ -337,7 +333,7 @@ def preprocess_X(X, distance, apply_pca, verbose, seed, high_dim):
         X /= xmax
         xmean = np.mean(X, axis=0)
         X -= xmean
-        tsvd = TruncatedSVD(n_components=2, random_state=seed) # for init only
+        tsvd = PCA(n_components=low_dim, random_state=seed) # for init only
         tsvd.fit(X)
         print_verbose("X is normalized", verbose)
     return X, pca_solution, tsvd
@@ -373,8 +369,8 @@ def distance_to_option(distance='euclidean'):
     elif distance == 'hamming':
         option = 3
     else:
-        raise NotImplementedError('Distance metric other than euclidean, manhattan,' +
-                                  'angular or hamming is not supported')
+        raise NotImplementedError('Distance other than euclidean, manhattan,' + \
+                    'angular or hamming is not supported')
     return option
 
 
@@ -458,7 +454,7 @@ def generate_pair(
     knn_distances = np.empty((n, n_neighbors_extra), dtype=np.float32)
 
     for i in range(n):
-        nbrs_ = tree.get_nns_by_item(i, n_neighbors_extra+1)
+        nbrs_ = tree.get_nns_by_item(i, n_neighbors_extra + 1)
         nbrs[i, :] = nbrs_[1:]
         for j in range(n_neighbors_extra):
             knn_distances[i, j] = tree.get_distance(i, nbrs[i, j])
@@ -497,8 +493,7 @@ def generate_pair_no_neighbors(
         pair_FP = sample_FP_pair(X, pair_neighbors, n_neighbors, n_FP)
     else:
         pair_MN = sample_MN_pair_deterministic(X, n_MN, _RANDOM_STATE, option)
-        pair_FP = sample_FP_pair_deterministic(
-            X, pair_neighbors, n_neighbors, n_FP, _RANDOM_STATE)
+        pair_FP = sample_FP_pair_deterministic(X, pair_neighbors, n_neighbors, n_FP, _RANDOM_STATE)
     return pair_neighbors, pair_MN, pair_FP
 
 
@@ -530,7 +525,7 @@ def pacmap(
         if pca_solution:
             Y = 0.01 * X[:, :n_dims]
         else:
-            Y = 0.01 * tsvd.fit_transform(X).astype(np.float32)
+            Y = 0.01 * tsvd.transform(X).astype(np.float32)
     elif Yinit == "random": # random or hamming distance
         if _RANDOM_STATE is not None:
             np.random.seed(_RANDOM_STATE)
@@ -665,7 +660,7 @@ class PaCMAP(BaseEstimator):
 
     Parameters
     ---------
-    n_dims: int, default=2
+    n_components: int, default=2
         Dimensions of the embedded space. We recommend to use 2 or 3.
     
     n_neighbors: int, default=10
@@ -721,7 +716,7 @@ class PaCMAP(BaseEstimator):
         Default to False for memory saving. Setting this option to True can make `transform()` method faster.
     '''
     def __init__(self,
-                 n_dims=2,
+                 n_components=2,
                  n_neighbors=10,
                  MN_ratio=0.5,
                  FP_ratio=2.0,
@@ -738,7 +733,7 @@ class PaCMAP(BaseEstimator):
                  random_state=None,
                  save_tree=False
                  ):
-        self.n_dims = n_dims
+        self.n_components = n_components
         self.n_neighbors = n_neighbors
         self.MN_ratio = MN_ratio
         self.FP_ratio = FP_ratio
@@ -769,13 +764,13 @@ class PaCMAP(BaseEstimator):
             self.random_state = 0
             _RANDOM_STATE = None  # Reset random state
 
-        if self.n_dims < 2:
+        if self.n_components < 2:
             raise ValueError(
                 "The number of projection dimensions must be at least 2.")
         if self.lr <= 0:
             raise ValueError("The learning rate must be larger than 0.")
         if self.distance == "hamming" and apply_pca:
-            warnings.warn("apply_pca = True for Hamming distance.")
+            warnings.warn("apply_pca = True for Hamming distance. This option will be ignored.")
         if not self.apply_pca:
             warnings.warn(
                 "Running ANNOY Indexing on high-dimensional data. Nearest-neighbor search may be slow!")
@@ -796,6 +791,23 @@ class PaCMAP(BaseEstimator):
                 "The number of further points can't be less than 1")
 
     def fit(self, X, init=None, save_pairs=True):
+        '''Projects a high dimensional dataset into a low-dimensional embedding, without returning the output.
+
+        Parameters
+        ---------
+        X: numpy.ndarray
+            The high-dimensional dataset that is being projected. 
+            An embedding will get created based on parameters of the PaCMAP instance.
+        
+        init: str, optional
+            One of ['pca', 'random']. Initialization of the embedding, default='pca'.
+            If 'pca', then the low dimensional embedding is initialized to the PCA mapped dataset.
+            If 'random', then the low dimensional embedding is initialized with a Gaussian distribution.
+        
+        save_pairs: bool, optional
+            Whether to save the pairs that are sampled from the dataset. Useful for reproducing results.
+        '''
+
         X = X.astype(np.float32)
         # Preprocess the dataset
         n, dim = X.shape
@@ -804,7 +816,7 @@ class PaCMAP(BaseEstimator):
         self.xmin = np.min(X)
         self.xmax = np.max(X)
         self.xmean = np.mean(X, axis=0)
-        X, pca_solution, tsvd = preprocess_X(X, self.distance, self.apply_pca, self.verbose, self.random_state, dim)
+        X, pca_solution, tsvd = preprocess_X(X, self.distance, self.apply_pca, self.verbose, self.random_state, dim, self.n_components)
         self.tsvd_transformer = tsvd
         self.pca_solution = pca_solution
         # Deciding the number of pairs
@@ -830,7 +842,7 @@ class PaCMAP(BaseEstimator):
         # Initialize and Optimize the embedding
         self.embedding_, self.intermediate_states, self.pair_neighbors, self.pair_MN, self.pair_FP = pacmap(
             X,
-            self.n_dims,
+            self.n_components,
             self.pair_neighbors,
             self.pair_MN,
             self.pair_FP,
@@ -848,6 +860,23 @@ class PaCMAP(BaseEstimator):
 
 
     def fit_transform(self, X, init=None, save_pairs=True):
+        '''Projects a high dimensional dataset into a low-dimensional embedding and return the embedding.
+
+        Parameters
+        ---------
+        X: numpy.ndarray
+            The high-dimensional dataset that is being projected. 
+            An embedding will get created based on parameters of the PaCMAP instance.
+        
+        init: str, optional
+            One of ['pca', 'random']. Initialization of the embedding, default='pca'.
+            If 'pca', then the low dimensional embedding is initialized to the PCA mapped dataset.
+            If 'random', then the low dimensional embedding is initialized with a Gaussian distribution.
+        
+        save_pairs: bool, optional
+            Whether to save the pairs that are sampled from the dataset. Useful for reproducing results.
+        '''
+
         self.fit(X, init, save_pairs)
         if self.intermediate:
             return self.intermediate_states
@@ -855,7 +884,29 @@ class PaCMAP(BaseEstimator):
             return self.embedding_
 
 
-    def transform(self, X, init=None, save_pairs=True, basis=None):
+    def transform(self, X, basis=None, init=None, save_pairs=True):
+        '''Projects a high dimensional dataset into existing embedding space and return the embedding.
+
+        Parameters
+        ---------
+        X: numpy.ndarray
+            The high-dimensional dataset that is being projected. 
+            An embedding will get created based on parameters of the PaCMAP instance.
+        
+        basis: numpy.ndarray
+            The original dataset that have already been `fit` of `fit_transform`ed.
+            If `save_tree == True`, then it's unnecessary to provide the original dataset again.
+
+        init: str, optional
+            One of ['pca', 'random']. Initialization of the embedding, default='pca'.
+            If 'pca', then the low dimensional embedding is initialized to the PCA mapped dataset. 
+            The PCA instance will be the same that transformed the original dataset. 
+            If 'random', then the low dimensional embedding is initialized with a Gaussian distribution.
+        
+        save_pairs: bool, optional
+            Whether to save the pairs that are sampled from the dataset. Useful for reproducing results.
+        '''
+
         # Preprocess the data
         X = X.astype(np.float32)
         X = preprocess_X_new(X, self.distance, self.xmin, self.xmax, 
@@ -872,8 +923,11 @@ class PaCMAP(BaseEstimator):
                             self.distance,
                             self.verbose
                             )
+        if not save_pairs:
+            self.pair_XP = None
+
         # Initialize and Optimize the embedding
-        Y, intermediate_states = pacmap_fit(X, self.embedding_, self.n_dims, self.pair_XP, self.lr, 
+        Y, intermediate_states = pacmap_fit(X, self.embedding_, self.n_components, self.pair_XP, self.lr, 
                                             self.num_iters, init, self.verbose, 
                                             self.intermediate, self.intermediate_snapshots,
                                             self.pca_solution, self.tsvd_transformer)
@@ -883,9 +937,19 @@ class PaCMAP(BaseEstimator):
             return Y[self.embedding_.shape[0]:, :]
 
 
-    def sample_pairs(self, X, save_tree=True):
+    def sample_pairs(self, X, save_tree):
+        '''
+        Sample PaCMAP pairs from the dataset.
+
+        Parameters
+        ---------
+        X: numpy.ndarray
+            The high-dimensional dataset that is being projected.
+
+        save_tree: bool
+            Whether to save the annoy index tree after finding the nearest neighbor pairs.
+        '''
         # Creating pairs
-        pca_solution = False
         print_verbose("Finding pairs", self.verbose)
         if self.pair_neighbors is None:
             self.pair_neighbors, self.pair_MN, self.pair_FP, self.tree = generate_pair(
@@ -909,6 +973,9 @@ class PaCMAP(BaseEstimator):
 
 
     def del_pairs(self):
+        '''
+        Delete stored pairs.
+        '''
         self.pair_neighbors = None
         self.pair_MN = None
         self.pair_FP = None
